from telethon.tl.mtproto_request import MTProtoRequest


class WebPage(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    webPage#ca820ed7 flags:None id:long url:string display_url:string type:flags.0?string site_name:flags.1?string title:flags.2?string description:flags.3?string photo:flags.4?Photo embed_url:flags.5?string embed_type:flags.5?string embed_width:flags.6?int embed_height:flags.6?int duration:flags.7?int author:flags.8?string document:flags.9?Document = WebPage"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0xca820ed7

    def __init__(self, id, url, display_url, type=None, site_name=None, title=None, description=None, photo=None, embed_url=None, embed_type=None, embed_width=None, embed_height=None, duration=None, author=None, document=None):
        """
        :param id: Telegram type: «long».
        :param url: Telegram type: «string».
        :param display_url: Telegram type: «string».
        :param type: Telegram type: «string».
        :param site_name: Telegram type: «string».
        :param title: Telegram type: «string».
        :param description: Telegram type: «string».
        :param photo: Telegram type: «Photo».
        :param embed_url: Telegram type: «string».
        :param embed_type: Telegram type: «string».
        :param embed_width: Telegram type: «int».
        :param embed_height: Telegram type: «int».
        :param duration: Telegram type: «int».
        :param author: Telegram type: «string».
        :param document: Telegram type: «Document».
        """
        super().__init__()

        self.id = id
        self.url = url
        self.display_url = display_url
        self.type = type
        self.site_name = site_name
        self.title = title
        self.description = description
        self.photo = photo
        self.embed_url = embed_url
        self.embed_type = embed_type
        self.embed_width = embed_width
        self.embed_height = embed_height
        self.duration = duration
        self.author = author
        self.document = document

    def on_send(self, writer):
        writer.write_int(WebPage.constructor_id, signed=False)
        # Calculate the flags. This equals to those flag arguments which are NOT None
        flags = 0
        flags |= (1 << 0) if self.type else 0
        flags |= (1 << 1) if self.site_name else 0
        flags |= (1 << 2) if self.title else 0
        flags |= (1 << 3) if self.description else 0
        flags |= (1 << 4) if self.photo else 0
        flags |= (1 << 5) if self.embed_url else 0
        flags |= (1 << 5) if self.embed_type else 0
        flags |= (1 << 6) if self.embed_width else 0
        flags |= (1 << 6) if self.embed_height else 0
        flags |= (1 << 7) if self.duration else 0
        flags |= (1 << 8) if self.author else 0
        flags |= (1 << 9) if self.document else 0
        writer.write_int(flags)

        writer.write_long(self.id)
        writer.tgwrite_string(self.url)
        writer.tgwrite_string(self.display_url)
        if self.type:
            writer.tgwrite_string(self.type)

        if self.site_name:
            writer.tgwrite_string(self.site_name)

        if self.title:
            writer.tgwrite_string(self.title)

        if self.description:
            writer.tgwrite_string(self.description)

        if self.photo:
            self.photo.on_send(writer)

        if self.embed_url:
            writer.tgwrite_string(self.embed_url)

        if self.embed_type:
            writer.tgwrite_string(self.embed_type)

        if self.embed_width:
            writer.write_int(self.embed_width)

        if self.embed_height:
            writer.write_int(self.embed_height)

        if self.duration:
            writer.write_int(self.duration)

        if self.author:
            writer.tgwrite_string(self.author)

        if self.document:
            self.document.on_send(writer)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return WebPage(None, None, None, None, None, None, None, None, None, None, None, None, None, None, None)

    def on_response(self, reader):
        flags = reader.read_int()

        self.id = reader.read_long()
        self.url = reader.tgread_string()
        self.display_url = reader.tgread_string()
        if (flags & (1 << 0)) != 0:
            self.type = reader.tgread_string()

        if (flags & (1 << 1)) != 0:
            self.site_name = reader.tgread_string()

        if (flags & (1 << 2)) != 0:
            self.title = reader.tgread_string()

        if (flags & (1 << 3)) != 0:
            self.description = reader.tgread_string()

        if (flags & (1 << 4)) != 0:
            self.photo = reader.tgread_object()

        if (flags & (1 << 5)) != 0:
            self.embed_url = reader.tgread_string()

        if (flags & (1 << 5)) != 0:
            self.embed_type = reader.tgread_string()

        if (flags & (1 << 6)) != 0:
            self.embed_width = reader.read_int()

        if (flags & (1 << 6)) != 0:
            self.embed_height = reader.read_int()

        if (flags & (1 << 7)) != 0:
            self.duration = reader.read_int()

        if (flags & (1 << 8)) != 0:
            self.author = reader.tgread_string()

        if (flags & (1 << 9)) != 0:
            self.document = reader.tgread_object()

    def __repr__(self):
        return 'webPage#ca820ed7 flags:None id:long url:string display_url:string type:flags.0?string site_name:flags.1?string title:flags.2?string description:flags.3?string photo:flags.4?Photo embed_url:flags.5?string embed_type:flags.5?string embed_width:flags.6?int embed_height:flags.6?int duration:flags.7?int author:flags.8?string document:flags.9?Document = WebPage'

    def __str__(self):
        return '(webPage (ID: 0xca820ed7) = (id={}, url={}, display_url={}, type={}, site_name={}, title={}, description={}, photo={}, embed_url={}, embed_type={}, embed_width={}, embed_height={}, duration={}, author={}, document={}))'.format(str(self.id), str(self.url), str(self.display_url), str(self.type), str(self.site_name), str(self.title), str(self.description), str(self.photo), str(self.embed_url), str(self.embed_type), str(self.embed_width), str(self.embed_height), str(self.duration), str(self.author), str(self.document))



from telethon.tl.mtproto_request import MTProtoRequest


class ReceivedNotifyMessage(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    receivedNotifyMessage#a384b779 id:int flags:int = ReceivedNotifyMessage"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0xa384b779

    def __init__(self, id, flags):
        """
        :param id: Telegram type: «int».
        :param flags: Telegram type: «int».
        """
        super().__init__()

        self.id = id
        self.flags = flags

    def on_send(self, writer):
        writer.write_int(ReceivedNotifyMessage.constructor_id, signed=False)
        writer.write_int(self.id)
        writer.write_int(self.flags)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return ReceivedNotifyMessage(None, None)

    def on_response(self, reader):
        self.id = reader.read_int()
        self.flags = reader.read_int()

    def __repr__(self):
        return 'receivedNotifyMessage#a384b779 id:int flags:int = ReceivedNotifyMessage'

    def __str__(self):
        return '(receivedNotifyMessage (ID: 0xa384b779) = (id={}, flags={}))'.format(str(self.id), str(self.flags))



from telethon.tl.mtproto_request import MTProtoRequest


class UpdateChannelTooLong(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    updateChannelTooLong#eb0467fb flags:None channel_id:int pts:flags.0?int = Update"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0xeb0467fb

    def __init__(self, channel_id, pts=None):
        """
        :param channel_id: Telegram type: «int».
        :param pts: Telegram type: «int».
        """
        super().__init__()

        self.channel_id = channel_id
        self.pts = pts

    def on_send(self, writer):
        writer.write_int(UpdateChannelTooLong.constructor_id, signed=False)
        # Calculate the flags. This equals to those flag arguments which are NOT None
        flags = 0
        flags |= (1 << 0) if self.pts else 0
        writer.write_int(flags)

        writer.write_int(self.channel_id)
        if self.pts:
            writer.write_int(self.pts)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return UpdateChannelTooLong(None, None)

    def on_response(self, reader):
        flags = reader.read_int()

        self.channel_id = reader.read_int()
        if (flags & (1 << 0)) != 0:
            self.pts = reader.read_int()

    def __repr__(self):
        return 'updateChannelTooLong#eb0467fb flags:None channel_id:int pts:flags.0?int = Update'

    def __str__(self):
        return '(updateChannelTooLong (ID: 0xeb0467fb) = (channel_id={}, pts={}))'.format(str(self.channel_id), str(self.pts))



from telethon.tl.mtproto_request import MTProtoRequest


class NoPassword(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    account.noPassword#96dabc18 new_salt:bytes email_unconfirmed_pattern:string = account.Password"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0x96dabc18

    def __init__(self, new_salt, email_unconfirmed_pattern):
        """
        :param new_salt: Telegram type: «bytes».
        :param email_unconfirmed_pattern: Telegram type: «string».
        """
        super().__init__()

        self.new_salt = new_salt
        self.email_unconfirmed_pattern = email_unconfirmed_pattern

    def on_send(self, writer):
        writer.write_int(NoPassword.constructor_id, signed=False)
        writer.tgwrite_bytes(self.new_salt)
        writer.tgwrite_string(self.email_unconfirmed_pattern)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return NoPassword(None, None)

    def on_response(self, reader):
        self.new_salt = reader.tgread_bytes()
        self.email_unconfirmed_pattern = reader.tgread_string()

    def __repr__(self):
        return 'account.noPassword#96dabc18 new_salt:bytes email_unconfirmed_pattern:string = account.Password'

    def __str__(self):
        return '(account.noPassword (ID: 0x96dabc18) = (new_salt={}, email_unconfirmed_pattern={}))'.format(str(self.new_salt), str(self.email_unconfirmed_pattern))



from telethon.tl.types.account.privacy_rules import PrivacyRules
from telethon.tl.types.account.authorizations import Authorizations
from telethon.tl.types.account.no_password import NoPassword
from telethon.tl.types.account.password import Password
from telethon.tl.types.account.password_settings import PasswordSettings
from telethon.tl.types.account.password_input_settings import PasswordInputSettings



from telethon.tl.mtproto_request import MTProtoRequest


class SaveGifRequest(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    messages.saveGif#327a30cb id:InputDocument unsave:Bool = Bool"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0x327a30cb

    def __init__(self, id, unsave):
        """
        :param id: Telegram type: «InputDocument».
        :param unsave: Telegram type: «Bool».
        """
        super().__init__()
        self.result = None
        self.confirmed = True  # Confirmed by default

        self.id = id
        self.unsave = unsave

    def on_send(self, writer):
        writer.write_int(SaveGifRequest.constructor_id, signed=False)
        self.id.on_send(writer)
        writer.tgwrite_bool(self.unsave)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return SaveGifRequest(None, None)

    def on_response(self, reader):
        self.result = reader.tgread_object()

    def __repr__(self):
        return 'messages.saveGif#327a30cb id:InputDocument unsave:Bool = Bool'

    def __str__(self):
        return '(messages.saveGif (ID: 0x327a30cb) = (id={}, unsave={}))'.format(str(self.id), str(self.unsave))



from telethon.tl.mtproto_request import MTProtoRequest


class UserProfilePhoto(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    userProfilePhoto#d559d8c8 photo_id:long photo_small:FileLocation photo_big:FileLocation = UserProfilePhoto"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0xd559d8c8

    def __init__(self, photo_id, photo_small, photo_big):
        """
        :param photo_id: Telegram type: «long».
        :param photo_small: Telegram type: «FileLocation».
        :param photo_big: Telegram type: «FileLocation».
        """
        super().__init__()

        self.photo_id = photo_id
        self.photo_small = photo_small
        self.photo_big = photo_big

    def on_send(self, writer):
        writer.write_int(UserProfilePhoto.constructor_id, signed=False)
        writer.write_long(self.photo_id)
        self.photo_small.on_send(writer)
        self.photo_big.on_send(writer)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return UserProfilePhoto(None, None, None)

    def on_response(self, reader):
        self.photo_id = reader.read_long()
        self.photo_small = reader.tgread_object()
        self.photo_big = reader.tgread_object()

    def __repr__(self):
        return 'userProfilePhoto#d559d8c8 photo_id:long photo_small:FileLocation photo_big:FileLocation = UserProfilePhoto'

    def __str__(self):
        return '(userProfilePhoto (ID: 0xd559d8c8) = (photo_id={}, photo_small={}, photo_big={}))'.format(str(self.photo_id), str(self.photo_small), str(self.photo_big))



from telethon.tl.mtproto_request import MTProtoRequest


class EditPhotoRequest(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    channels.editPhoto#f12e57c9 channel:InputChannel photo:InputChatPhoto = Updates"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0xf12e57c9

    def __init__(self, channel, photo):
        """
        :param channel: Telegram type: «InputChannel».
        :param photo: Telegram type: «InputChatPhoto».
        """
        super().__init__()
        self.result = None
        self.confirmed = True  # Confirmed by default

        self.channel = channel
        self.photo = photo

    def on_send(self, writer):
        writer.write_int(EditPhotoRequest.constructor_id, signed=False)
        self.channel.on_send(writer)
        self.photo.on_send(writer)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return EditPhotoRequest(None, None)

    def on_response(self, reader):
        self.result = reader.tgread_object()

    def __repr__(self):
        return 'channels.editPhoto#f12e57c9 channel:InputChannel photo:InputChatPhoto = Updates'

    def __str__(self):
        return '(channels.editPhoto (ID: 0xf12e57c9) = (channel={}, photo={}))'.format(str(self.channel), str(self.photo))



from telethon.tl.mtproto_request import MTProtoRequest


class PasswordSettings(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    account.passwordSettings#b7b72ab3 email:string = account.PasswordSettings"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0xb7b72ab3

    def __init__(self, email):
        """
        :param email: Telegram type: «string».
        """
        super().__init__()

        self.email = email

    def on_send(self, writer):
        writer.write_int(PasswordSettings.constructor_id, signed=False)
        writer.tgwrite_string(self.email)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return PasswordSettings(None)

    def on_response(self, reader):
        self.email = reader.tgread_string()

    def __repr__(self):
        return 'account.passwordSettings#b7b72ab3 email:string = account.PasswordSettings'

    def __str__(self):
        return '(account.passwordSettings (ID: 0xb7b72ab3) = (email={}))'.format(str(self.email))



from telethon.tl.mtproto_request import MTProtoRequest


class UpdateReadChannelOutbox(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    updateReadChannelOutbox#25d6c9c7 channel_id:int max_id:int = Update"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0x25d6c9c7

    def __init__(self, channel_id, max_id):
        """
        :param channel_id: Telegram type: «int».
        :param max_id: Telegram type: «int».
        """
        super().__init__()

        self.channel_id = channel_id
        self.max_id = max_id

    def on_send(self, writer):
        writer.write_int(UpdateReadChannelOutbox.constructor_id, signed=False)
        writer.write_int(self.channel_id)
        writer.write_int(self.max_id)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return UpdateReadChannelOutbox(None, None)

    def on_response(self, reader):
        self.channel_id = reader.read_int()
        self.max_id = reader.read_int()

    def __repr__(self):
        return 'updateReadChannelOutbox#25d6c9c7 channel_id:int max_id:int = Update'

    def __str__(self):
        return '(updateReadChannelOutbox (ID: 0x25d6c9c7) = (channel_id={}, max_id={}))'.format(str(self.channel_id), str(self.max_id))



from telethon.tl.mtproto_request import MTProtoRequest


class ImportCardRequest(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    contacts.importCard#4fe196fe export_card:Vector<int> = User"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0x4fe196fe

    def __init__(self, export_card):
        """
        :param export_card: Telegram type: «int». Must be a list.
        """
        super().__init__()
        self.result = None
        self.confirmed = True  # Confirmed by default

        self.export_card = export_card

    def on_send(self, writer):
        writer.write_int(ImportCardRequest.constructor_id, signed=False)
        writer.write_int(0x1cb5c415, signed=False)  # Vector's constructor ID
        writer.write_int(len(self.export_card))
        for export_card_item in self.export_card:
            writer.write_int(export_card_item)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return ImportCardRequest(None)

    def on_response(self, reader):
        self.result = reader.tgread_object()

    def __repr__(self):
        return 'contacts.importCard#4fe196fe export_card:Vector<int> = User'

    def __str__(self):
        return '(contacts.importCard (ID: 0x4fe196fe) = (export_card={}))'.format(None if not self.export_card else [str(_) for _ in self.export_card])



from telethon.tl.mtproto_request import MTProtoRequest


class ChatInvite(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    chatInvite#db74f558 flags:None channel:flags.0?true broadcast:flags.1?true public:flags.2?true megagroup:flags.3?true title:string photo:ChatPhoto participants_count:int participants:flags.4?Vector<User> = ChatInvite"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0xdb74f558

    def __init__(self, title, photo, participants_count, channel=None, broadcast=None, public=None, megagroup=None, participants=None):
        """
        :param channel: Telegram type: «true».
        :param broadcast: Telegram type: «true».
        :param public: Telegram type: «true».
        :param megagroup: Telegram type: «true».
        :param title: Telegram type: «string».
        :param photo: Telegram type: «ChatPhoto».
        :param participants_count: Telegram type: «int».
        :param participants: Telegram type: «User». Must be a list.
        """
        super().__init__()

        self.channel = channel
        self.broadcast = broadcast
        self.public = public
        self.megagroup = megagroup
        self.title = title
        self.photo = photo
        self.participants_count = participants_count
        self.participants = participants

    def on_send(self, writer):
        writer.write_int(ChatInvite.constructor_id, signed=False)
        # Calculate the flags. This equals to those flag arguments which are NOT None
        flags = 0
        flags |= (1 << 0) if self.channel else 0
        flags |= (1 << 1) if self.broadcast else 0
        flags |= (1 << 2) if self.public else 0
        flags |= (1 << 3) if self.megagroup else 0
        flags |= (1 << 4) if self.participants else 0
        writer.write_int(flags)

        writer.tgwrite_string(self.title)
        self.photo.on_send(writer)
        writer.write_int(self.participants_count)
        if self.participants:
            writer.write_int(0x1cb5c415, signed=False)  # Vector's constructor ID
            writer.write_int(len(self.participants))
            for participants_item in self.participants:
                if participants_item:
                    participants_item.on_send(writer)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return ChatInvite(None, None, None, None, None, None, None, None)

    def on_response(self, reader):
        flags = reader.read_int()

        if (flags & (1 << 0)) != 0:
            self.channel = True  # Arbitrary not-None value, no need to read since it is a flag

        if (flags & (1 << 1)) != 0:
            self.broadcast = True  # Arbitrary not-None value, no need to read since it is a flag

        if (flags & (1 << 2)) != 0:
            self.public = True  # Arbitrary not-None value, no need to read since it is a flag

        if (flags & (1 << 3)) != 0:
            self.megagroup = True  # Arbitrary not-None value, no need to read since it is a flag

        self.title = reader.tgread_string()
        self.photo = reader.tgread_object()
        self.participants_count = reader.read_int()
        if (flags & (1 << 4)) != 0:
            reader.read_int()  # Vector's constructor ID
            self.participants = []  # Initialize an empty list
            participants_len = reader.read_int()
            for _ in range(participants_len):
                participants_item = reader.tgread_object()
                self.participants.append(participants_item)

    def __repr__(self):
        return 'chatInvite#db74f558 flags:None channel:flags.0?true broadcast:flags.1?true public:flags.2?true megagroup:flags.3?true title:string photo:ChatPhoto participants_count:int participants:flags.4?Vector<User> = ChatInvite'

    def __str__(self):
        return '(chatInvite (ID: 0xdb74f558) = (channel={}, broadcast={}, public={}, megagroup={}, title={}, photo={}, participants_count={}, participants={}))'.format(str(self.channel), str(self.broadcast), str(self.public), str(self.megagroup), str(self.title), str(self.photo), str(self.participants_count), None if not self.participants else [str(_) for _ in self.participants])



from telethon.tl.mtproto_request import MTProtoRequest


class UpdateChatParticipants(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    updateChatParticipants#07761198 participants:ChatParticipants = Update"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0x7761198

    def __init__(self, participants):
        """
        :param participants: Telegram type: «ChatParticipants».
        """
        super().__init__()

        self.participants = participants

    def on_send(self, writer):
        writer.write_int(UpdateChatParticipants.constructor_id, signed=False)
        self.participants.on_send(writer)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return UpdateChatParticipants(None)

    def on_response(self, reader):
        self.participants = reader.tgread_object()

    def __repr__(self):
        return 'updateChatParticipants#07761198 participants:ChatParticipants = Update'

    def __str__(self):
        return '(updateChatParticipants (ID: 0x7761198) = (participants={}))'.format(str(self.participants))



from telethon.tl.mtproto_request import MTProtoRequest


class UpdateReadHistoryOutbox(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    updateReadHistoryOutbox#2f2f21bf peer:Peer max_id:int pts:int pts_count:int = Update"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0x2f2f21bf

    def __init__(self, peer, max_id, pts, pts_count):
        """
        :param peer: Telegram type: «Peer».
        :param max_id: Telegram type: «int».
        :param pts: Telegram type: «int».
        :param pts_count: Telegram type: «int».
        """
        super().__init__()

        self.peer = peer
        self.max_id = max_id
        self.pts = pts
        self.pts_count = pts_count

    def on_send(self, writer):
        writer.write_int(UpdateReadHistoryOutbox.constructor_id, signed=False)
        self.peer.on_send(writer)
        writer.write_int(self.max_id)
        writer.write_int(self.pts)
        writer.write_int(self.pts_count)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return UpdateReadHistoryOutbox(None, None, None, None)

    def on_response(self, reader):
        self.peer = reader.tgread_object()
        self.max_id = reader.read_int()
        self.pts = reader.read_int()
        self.pts_count = reader.read_int()

    def __repr__(self):
        return 'updateReadHistoryOutbox#2f2f21bf peer:Peer max_id:int pts:int pts_count:int = Update'

    def __str__(self):
        return '(updateReadHistoryOutbox (ID: 0x2f2f21bf) = (peer={}, max_id={}, pts={}, pts_count={}))'.format(str(self.peer), str(self.max_id), str(self.pts), str(self.pts_count))



from telethon.tl.mtproto_request import MTProtoRequest


class Contact(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    contact#f911c994 user_id:int mutual:Bool = Contact"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0xf911c994

    def __init__(self, user_id, mutual):
        """
        :param user_id: Telegram type: «int».
        :param mutual: Telegram type: «Bool».
        """
        super().__init__()

        self.user_id = user_id
        self.mutual = mutual

    def on_send(self, writer):
        writer.write_int(Contact.constructor_id, signed=False)
        writer.write_int(self.user_id)
        writer.tgwrite_bool(self.mutual)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return Contact(None, None)

    def on_response(self, reader):
        self.user_id = reader.read_int()
        self.mutual = reader.tgread_bool()

    def __repr__(self):
        return 'contact#f911c994 user_id:int mutual:Bool = Contact'

    def __str__(self):
        return '(contact (ID: 0xf911c994) = (user_id={}, mutual={}))'.format(str(self.user_id), str(self.mutual))



from telethon.tl.mtproto_request import MTProtoRequest


class GetDocumentByHashRequest(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    messages.getDocumentByHash#338e2464 sha256:bytes size:int mime_type:string = Document"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0x338e2464

    def __init__(self, sha256, size, mime_type):
        """
        :param sha256: Telegram type: «bytes».
        :param size: Telegram type: «int».
        :param mime_type: Telegram type: «string».
        """
        super().__init__()
        self.result = None
        self.confirmed = True  # Confirmed by default

        self.sha256 = sha256
        self.size = size
        self.mime_type = mime_type

    def on_send(self, writer):
        writer.write_int(GetDocumentByHashRequest.constructor_id, signed=False)
        writer.tgwrite_bytes(self.sha256)
        writer.write_int(self.size)
        writer.tgwrite_string(self.mime_type)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return GetDocumentByHashRequest(None, None, None)

    def on_response(self, reader):
        self.result = reader.tgread_object()

    def __repr__(self):
        return 'messages.getDocumentByHash#338e2464 sha256:bytes size:int mime_type:string = Document'

    def __str__(self):
        return '(messages.getDocumentByHash (ID: 0x338e2464) = (sha256={}, size={}, mime_type={}))'.format(str(self.sha256), str(self.size), str(self.mime_type))



from telethon.tl.mtproto_request import MTProtoRequest


class Game(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    game#bdf9653b flags:None id:long access_hash:long short_name:string title:string description:string photo:Photo document:flags.0?Document = Game"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0xbdf9653b

    def __init__(self, id, access_hash, short_name, title, description, photo, document=None):
        """
        :param id: Telegram type: «long».
        :param access_hash: Telegram type: «long».
        :param short_name: Telegram type: «string».
        :param title: Telegram type: «string».
        :param description: Telegram type: «string».
        :param photo: Telegram type: «Photo».
        :param document: Telegram type: «Document».
        """
        super().__init__()

        self.id = id
        self.access_hash = access_hash
        self.short_name = short_name
        self.title = title
        self.description = description
        self.photo = photo
        self.document = document

    def on_send(self, writer):
        writer.write_int(Game.constructor_id, signed=False)
        # Calculate the flags. This equals to those flag arguments which are NOT None
        flags = 0
        flags |= (1 << 0) if self.document else 0
        writer.write_int(flags)

        writer.write_long(self.id)
        writer.write_long(self.access_hash)
        writer.tgwrite_string(self.short_name)
        writer.tgwrite_string(self.title)
        writer.tgwrite_string(self.description)
        self.photo.on_send(writer)
        if self.document:
            self.document.on_send(writer)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return Game(None, None, None, None, None, None, None)

    def on_response(self, reader):
        flags = reader.read_int()

        self.id = reader.read_long()
        self.access_hash = reader.read_long()
        self.short_name = reader.tgread_string()
        self.title = reader.tgread_string()
        self.description = reader.tgread_string()
        self.photo = reader.tgread_object()
        if (flags & (1 << 0)) != 0:
            self.document = reader.tgread_object()

    def __repr__(self):
        return 'game#bdf9653b flags:None id:long access_hash:long short_name:string title:string description:string photo:Photo document:flags.0?Document = Game'

    def __str__(self):
        return '(game (ID: 0xbdf9653b) = (id={}, access_hash={}, short_name={}, title={}, description={}, photo={}, document={}))'.format(str(self.id), str(self.access_hash), str(self.short_name), str(self.title), str(self.description), str(self.photo), str(self.document))



from telethon.tl.mtproto_request import MTProtoRequest


class DiscardEncryptionRequest(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    messages.discardEncryption#edd923c5 chat_id:int = Bool"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0xedd923c5

    def __init__(self, chat_id):
        """
        :param chat_id: Telegram type: «int».
        """
        super().__init__()
        self.result = None
        self.confirmed = True  # Confirmed by default

        self.chat_id = chat_id

    def on_send(self, writer):
        writer.write_int(DiscardEncryptionRequest.constructor_id, signed=False)
        writer.write_int(self.chat_id)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return DiscardEncryptionRequest(None)

    def on_response(self, reader):
        self.result = reader.tgread_object()

    def __repr__(self):
        return 'messages.discardEncryption#edd923c5 chat_id:int = Bool'

    def __str__(self):
        return '(messages.discardEncryption (ID: 0xedd923c5) = (chat_id={}))'.format(str(self.chat_id))



from telethon.tl.mtproto_request import MTProtoRequest


class UpdateContactLink(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    updateContactLink#9d2e67c5 user_id:int my_link:ContactLink foreign_link:ContactLink = Update"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0x9d2e67c5

    def __init__(self, user_id, my_link, foreign_link):
        """
        :param user_id: Telegram type: «int».
        :param my_link: Telegram type: «ContactLink».
        :param foreign_link: Telegram type: «ContactLink».
        """
        super().__init__()

        self.user_id = user_id
        self.my_link = my_link
        self.foreign_link = foreign_link

    def on_send(self, writer):
        writer.write_int(UpdateContactLink.constructor_id, signed=False)
        writer.write_int(self.user_id)
        self.my_link.on_send(writer)
        self.foreign_link.on_send(writer)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return UpdateContactLink(None, None, None)

    def on_response(self, reader):
        self.user_id = reader.read_int()
        self.my_link = reader.tgread_object()
        self.foreign_link = reader.tgread_object()

    def __repr__(self):
        return 'updateContactLink#9d2e67c5 user_id:int my_link:ContactLink foreign_link:ContactLink = Update'

    def __str__(self):
        return '(updateContactLink (ID: 0x9d2e67c5) = (user_id={}, my_link={}, foreign_link={}))'.format(str(self.user_id), str(self.my_link), str(self.foreign_link))



from telethon.tl.mtproto_request import MTProtoRequest


class SetEncryptedTypingRequest(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    messages.setEncryptedTyping#791451ed peer:InputEncryptedChat typing:Bool = Bool"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0x791451ed

    def __init__(self, peer, typing):
        """
        :param peer: Telegram type: «InputEncryptedChat».
        :param typing: Telegram type: «Bool».
        """
        super().__init__()
        self.result = None
        self.confirmed = True  # Confirmed by default

        self.peer = peer
        self.typing = typing

    def on_send(self, writer):
        writer.write_int(SetEncryptedTypingRequest.constructor_id, signed=False)
        self.peer.on_send(writer)
        writer.tgwrite_bool(self.typing)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return SetEncryptedTypingRequest(None, None)

    def on_response(self, reader):
        self.result = reader.tgread_object()

    def __repr__(self):
        return 'messages.setEncryptedTyping#791451ed peer:InputEncryptedChat typing:Bool = Bool'

    def __str__(self):
        return '(messages.setEncryptedTyping (ID: 0x791451ed) = (peer={}, typing={}))'.format(str(self.peer), str(self.typing))



from os.path import isfile as file_exists
import os
import time
import pickle
import random
import telethon.helpers as utils


class Session:
    def __init__(self, session_user_id):
        self.session_user_id = session_user_id
        self.server_address = '91.108.56.165'
        self.port = 443
        self.auth_key = None
        self.id = utils.generate_random_long(signed=False)
        self.sequence = 0
        self.salt = 0  # Unsigned long
        self.time_offset = 0
        self.last_message_id = 0  # Long
        self.user = None

    def save(self):
        """Saves the current session object as session_user_id.session"""
        if self.session_user_id:
            with open('{}.session'.format(self.session_user_id), 'wb') as file:
                pickle.dump(self, file)

    def delete(self):
        """Deletes the current session file"""
        try:
            os.remove('{}.session'.format(self.session_user_id))
            return True
        except:
            return False

    @staticmethod
    def try_load_or_create_new(session_user_id):
        """Loads a saved session_user_id session, or creates a new one if none existed before.
           If the given session_user_id is None, we assume that it is for testing purposes"""
        if session_user_id is None:
            return Session(None)

        else:
            filepath = '{}.session'.format(session_user_id)

            if file_exists(filepath):
                with open(filepath, 'rb') as file:
                    return pickle.load(file)
            else:
                return Session(session_user_id)

    def get_new_msg_id(self):
        """Generates a new message ID based on the current time (in ms) since epoch"""
        # Refer to mtproto_plain_sender.py for the original method, this is a simple copy
        ms_time = int(time.time() * 1000)
        new_msg_id = (((ms_time // 1000 + self.time_offset) << 32) |  # "must approximately equal unixtime*2^32"
                      ((ms_time % 1000) << 22) |  # "approximate moment in time the message was created"
                      random.randint(0, 524288) << 2)  # "message identifiers are divisible by 4"

        if self.last_message_id >= new_msg_id:
            new_msg_id = self.last_message_id + 4

        self.last_message_id = new_msg_id
        return new_msg_id



from telethon.tl.mtproto_request import MTProtoRequest


class SaveRecentStickerRequest(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    messages.saveRecentSticker#392718f8 flags:None attached:flags.0?true id:InputDocument unsave:Bool = Bool"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0x392718f8

    def __init__(self, id, unsave, attached=None):
        """
        :param attached: Telegram type: «true».
        :param id: Telegram type: «InputDocument».
        :param unsave: Telegram type: «Bool».
        """
        super().__init__()
        self.result = None
        self.confirmed = True  # Confirmed by default

        self.attached = attached
        self.id = id
        self.unsave = unsave

    def on_send(self, writer):
        writer.write_int(SaveRecentStickerRequest.constructor_id, signed=False)
        # Calculate the flags. This equals to those flag arguments which are NOT None
        flags = 0
        flags |= (1 << 0) if self.attached else 0
        writer.write_int(flags)

        self.id.on_send(writer)
        writer.tgwrite_bool(self.unsave)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return SaveRecentStickerRequest(None, None, None)

    def on_response(self, reader):
        self.result = reader.tgread_object()

    def __repr__(self):
        return 'messages.saveRecentSticker#392718f8 flags:None attached:flags.0?true id:InputDocument unsave:Bool = Bool'

    def __str__(self):
        return '(messages.saveRecentSticker (ID: 0x392718f8) = (attached={}, id={}, unsave={}))'.format(str(self.attached), str(self.id), str(self.unsave))



from telethon.tl.functions.auth.check_phone import CheckPhoneRequest
from telethon.tl.functions.auth.send_code import SendCodeRequest
from telethon.tl.functions.auth.sign_up import SignUpRequest
from telethon.tl.functions.auth.sign_in import SignInRequest
from telethon.tl.functions.auth.log_out import LogOutRequest
from telethon.tl.functions.auth.reset_authorizations import ResetAuthorizationsRequest
from telethon.tl.functions.auth.send_invites import SendInvitesRequest
from telethon.tl.functions.auth.export_authorization import ExportAuthorizationRequest
from telethon.tl.functions.auth.import_authorization import ImportAuthorizationRequest
from telethon.tl.functions.auth.bind_temp_auth_key import BindTempAuthKeyRequest
from telethon.tl.functions.auth.import_bot_authorization import ImportBotAuthorizationRequest
from telethon.tl.functions.auth.check_password import CheckPasswordRequest
from telethon.tl.functions.auth.request_password_recovery import RequestPasswordRecoveryRequest
from telethon.tl.functions.auth.recover_password import RecoverPasswordRequest
from telethon.tl.functions.auth.resend_code import ResendCodeRequest
from telethon.tl.functions.auth.cancel_code import CancelCodeRequest
from telethon.tl.functions.auth.drop_temp_auth_keys import DropTempAuthKeysRequest



from telethon.tl.mtproto_request import MTProtoRequest


class GeoPoint(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    geoPoint#2049d70c long:double lat:double = GeoPoint"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0x2049d70c

    def __init__(self, long, lat):
        """
        :param long: Telegram type: «double».
        :param lat: Telegram type: «double».
        """
        super().__init__()

        self.long = long
        self.lat = lat

    def on_send(self, writer):
        writer.write_int(GeoPoint.constructor_id, signed=False)
        writer.write_double(self.long)
        writer.write_double(self.lat)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return GeoPoint(None, None)

    def on_response(self, reader):
        self.long = reader.read_double()
        self.lat = reader.read_double()

    def __repr__(self):
        return 'geoPoint#2049d70c long:double lat:double = GeoPoint'

    def __str__(self):
        return '(geoPoint (ID: 0x2049d70c) = (long={}, lat={}))'.format(str(self.long), str(self.lat))



from telethon.tl.mtproto_request import MTProtoRequest


class MessageActionChatJoinedByLink(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    messageActionChatJoinedByLink#f89cf5e8 inviter_id:int = MessageAction"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0xf89cf5e8

    def __init__(self, inviter_id):
        """
        :param inviter_id: Telegram type: «int».
        """
        super().__init__()

        self.inviter_id = inviter_id

    def on_send(self, writer):
        writer.write_int(MessageActionChatJoinedByLink.constructor_id, signed=False)
        writer.write_int(self.inviter_id)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return MessageActionChatJoinedByLink(None)

    def on_response(self, reader):
        self.inviter_id = reader.read_int()

    def __repr__(self):
        return 'messageActionChatJoinedByLink#f89cf5e8 inviter_id:int = MessageAction'

    def __str__(self):
        return '(messageActionChatJoinedByLink (ID: 0xf89cf5e8) = (inviter_id={}))'.format(str(self.inviter_id))



from telethon.tl.mtproto_request import MTProtoRequest


class InputFileLocation(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    inputFileLocation#14637196 volume_id:long local_id:int secret:long = InputFileLocation"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0x14637196

    def __init__(self, volume_id, local_id, secret):
        """
        :param volume_id: Telegram type: «long».
        :param local_id: Telegram type: «int».
        :param secret: Telegram type: «long».
        """
        super().__init__()

        self.volume_id = volume_id
        self.local_id = local_id
        self.secret = secret

    def on_send(self, writer):
        writer.write_int(InputFileLocation.constructor_id, signed=False)
        writer.write_long(self.volume_id)
        writer.write_int(self.local_id)
        writer.write_long(self.secret)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return InputFileLocation(None, None, None)

    def on_response(self, reader):
        self.volume_id = reader.read_long()
        self.local_id = reader.read_int()
        self.secret = reader.read_long()

    def __repr__(self):
        return 'inputFileLocation#14637196 volume_id:long local_id:int secret:long = InputFileLocation'

    def __str__(self):
        return '(inputFileLocation (ID: 0x14637196) = (volume_id={}, local_id={}, secret={}))'.format(str(self.volume_id), str(self.local_id), str(self.secret))



from telethon.tl.mtproto_request import MTProtoRequest


class PasswordRecovery(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    auth.passwordRecovery#137948a5 email_pattern:string = auth.PasswordRecovery"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0x137948a5

    def __init__(self, email_pattern):
        """
        :param email_pattern: Telegram type: «string».
        """
        super().__init__()

        self.email_pattern = email_pattern

    def on_send(self, writer):
        writer.write_int(PasswordRecovery.constructor_id, signed=False)
        writer.tgwrite_string(self.email_pattern)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return PasswordRecovery(None)

    def on_response(self, reader):
        self.email_pattern = reader.tgread_string()

    def __repr__(self):
        return 'auth.passwordRecovery#137948a5 email_pattern:string = auth.PasswordRecovery'

    def __str__(self):
        return '(auth.passwordRecovery (ID: 0x137948a5) = (email_pattern={}))'.format(str(self.email_pattern))



from telethon.tl.mtproto_request import MTProtoRequest


class UpdatePinnedMessageRequest(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    channels.updatePinnedMessage#a72ded52 flags:None silent:flags.0?true channel:InputChannel id:int = Updates"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0xa72ded52

    def __init__(self, channel, id, silent=None):
        """
        :param silent: Telegram type: «true».
        :param channel: Telegram type: «InputChannel».
        :param id: Telegram type: «int».
        """
        super().__init__()
        self.result = None
        self.confirmed = True  # Confirmed by default

        self.silent = silent
        self.channel = channel
        self.id = id

    def on_send(self, writer):
        writer.write_int(UpdatePinnedMessageRequest.constructor_id, signed=False)
        # Calculate the flags. This equals to those flag arguments which are NOT None
        flags = 0
        flags |= (1 << 0) if self.silent else 0
        writer.write_int(flags)

        self.channel.on_send(writer)
        writer.write_int(self.id)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return UpdatePinnedMessageRequest(None, None, None)

    def on_response(self, reader):
        self.result = reader.tgread_object()

    def __repr__(self):
        return 'channels.updatePinnedMessage#a72ded52 flags:None silent:flags.0?true channel:InputChannel id:int = Updates'

    def __str__(self):
        return '(channels.updatePinnedMessage (ID: 0xa72ded52) = (silent={}, channel={}, id={}))'.format(str(self.silent), str(self.channel), str(self.id))



from telethon.tl.mtproto_request import MTProtoRequest


class DocumentEmpty(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    documentEmpty#36f8c871 id:long = Document"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0x36f8c871

    def __init__(self, id):
        """
        :param id: Telegram type: «long».
        """
        super().__init__()

        self.id = id

    def on_send(self, writer):
        writer.write_int(DocumentEmpty.constructor_id, signed=False)
        writer.write_long(self.id)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return DocumentEmpty(None)

    def on_response(self, reader):
        self.id = reader.read_long()

    def __repr__(self):
        return 'documentEmpty#36f8c871 id:long = Document'

    def __str__(self):
        return '(documentEmpty (ID: 0x36f8c871) = (id={}))'.format(str(self.id))



from telethon.tl.mtproto_request import MTProtoRequest


class UpdateNotifySettings(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    updateNotifySettings#bec268ef peer:NotifyPeer notify_settings:PeerNotifySettings = Update"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0xbec268ef

    def __init__(self, peer, notify_settings):
        """
        :param peer: Telegram type: «NotifyPeer».
        :param notify_settings: Telegram type: «PeerNotifySettings».
        """
        super().__init__()

        self.peer = peer
        self.notify_settings = notify_settings

    def on_send(self, writer):
        writer.write_int(UpdateNotifySettings.constructor_id, signed=False)
        self.peer.on_send(writer)
        self.notify_settings.on_send(writer)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return UpdateNotifySettings(None, None)

    def on_response(self, reader):
        self.peer = reader.tgread_object()
        self.notify_settings = reader.tgread_object()

    def __repr__(self):
        return 'updateNotifySettings#bec268ef peer:NotifyPeer notify_settings:PeerNotifySettings = Update'

    def __str__(self):
        return '(updateNotifySettings (ID: 0xbec268ef) = (peer={}, notify_settings={}))'.format(str(self.peer), str(self.notify_settings))



from telethon.tl.mtproto_request import MTProtoRequest


class MessageMediaPhoto(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    messageMediaPhoto#3d8ce53d photo:Photo caption:string = MessageMedia"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0x3d8ce53d

    def __init__(self, photo, caption):
        """
        :param photo: Telegram type: «Photo».
        :param caption: Telegram type: «string».
        """
        super().__init__()

        self.photo = photo
        self.caption = caption

    def on_send(self, writer):
        writer.write_int(MessageMediaPhoto.constructor_id, signed=False)
        self.photo.on_send(writer)
        writer.tgwrite_string(self.caption)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return MessageMediaPhoto(None, None)

    def on_response(self, reader):
        self.photo = reader.tgread_object()
        self.caption = reader.tgread_string()

    def __repr__(self):
        return 'messageMediaPhoto#3d8ce53d photo:Photo caption:string = MessageMedia'

    def __str__(self):
        return '(messageMediaPhoto (ID: 0x3d8ce53d) = (photo={}, caption={}))'.format(str(self.photo), str(self.caption))



from telethon.tl.mtproto_request import MTProtoRequest


class UpdateUserPhoto(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    updateUserPhoto#95313b0c user_id:int date:date photo:UserProfilePhoto previous:Bool = Update"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0x95313b0c

    def __init__(self, user_id, date, photo, previous):
        """
        :param user_id: Telegram type: «int».
        :param date: Telegram type: «date».
        :param photo: Telegram type: «UserProfilePhoto».
        :param previous: Telegram type: «Bool».
        """
        super().__init__()

        self.user_id = user_id
        self.date = date
        self.photo = photo
        self.previous = previous

    def on_send(self, writer):
        writer.write_int(UpdateUserPhoto.constructor_id, signed=False)
        writer.write_int(self.user_id)
        writer.tgwrite_date(self.date)
        self.photo.on_send(writer)
        writer.tgwrite_bool(self.previous)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return UpdateUserPhoto(None, None, None, None)

    def on_response(self, reader):
        self.user_id = reader.read_int()
        self.date = reader.tgread_date()
        self.photo = reader.tgread_object()
        self.previous = reader.tgread_bool()

    def __repr__(self):
        return 'updateUserPhoto#95313b0c user_id:int date:date photo:UserProfilePhoto previous:Bool = Update'

    def __str__(self):
        return '(updateUserPhoto (ID: 0x95313b0c) = (user_id={}, date={}, photo={}, previous={}))'.format(str(self.user_id), str(self.date), str(self.photo), str(self.previous))



from telethon.tl.mtproto_request import MTProtoRequest


class CheckPasswordRequest(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    auth.checkPassword#0a63011e password_hash:bytes = auth.Authorization"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0xa63011e

    def __init__(self, password_hash):
        """
        :param password_hash: Telegram type: «bytes».
        """
        super().__init__()
        self.result = None
        self.confirmed = True  # Confirmed by default

        self.password_hash = password_hash

    def on_send(self, writer):
        writer.write_int(CheckPasswordRequest.constructor_id, signed=False)
        writer.tgwrite_bytes(self.password_hash)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return CheckPasswordRequest(None)

    def on_response(self, reader):
        self.result = reader.tgread_object()

    def __repr__(self):
        return 'auth.checkPassword#0a63011e password_hash:bytes = auth.Authorization'

    def __str__(self):
        return '(auth.checkPassword (ID: 0xa63011e) = (password_hash={}))'.format(str(self.password_hash))



from telethon.tl.mtproto_request import MTProtoRequest


class ReplyKeyboardMarkup(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    replyKeyboardMarkup#3502758c flags:None resize:flags.0?true single_use:flags.1?true selective:flags.2?true rows:Vector<KeyboardButtonRow> = ReplyMarkup"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0x3502758c

    def __init__(self, rows, resize=None, single_use=None, selective=None):
        """
        :param resize: Telegram type: «true».
        :param single_use: Telegram type: «true».
        :param selective: Telegram type: «true».
        :param rows: Telegram type: «KeyboardButtonRow». Must be a list.
        """
        super().__init__()

        self.resize = resize
        self.single_use = single_use
        self.selective = selective
        self.rows = rows

    def on_send(self, writer):
        writer.write_int(ReplyKeyboardMarkup.constructor_id, signed=False)
        # Calculate the flags. This equals to those flag arguments which are NOT None
        flags = 0
        flags |= (1 << 0) if self.resize else 0
        flags |= (1 << 1) if self.single_use else 0
        flags |= (1 << 2) if self.selective else 0
        writer.write_int(flags)

        writer.write_int(0x1cb5c415, signed=False)  # Vector's constructor ID
        writer.write_int(len(self.rows))
        for rows_item in self.rows:
            rows_item.on_send(writer)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return ReplyKeyboardMarkup(None, None, None, None)

    def on_response(self, reader):
        flags = reader.read_int()

        if (flags & (1 << 0)) != 0:
            self.resize = True  # Arbitrary not-None value, no need to read since it is a flag

        if (flags & (1 << 1)) != 0:
            self.single_use = True  # Arbitrary not-None value, no need to read since it is a flag

        if (flags & (1 << 2)) != 0:
            self.selective = True  # Arbitrary not-None value, no need to read since it is a flag

        reader.read_int()  # Vector's constructor ID
        self.rows = []  # Initialize an empty list
        rows_len = reader.read_int()
        for _ in range(rows_len):
            rows_item = reader.tgread_object()
            self.rows.append(rows_item)

    def __repr__(self):
        return 'replyKeyboardMarkup#3502758c flags:None resize:flags.0?true single_use:flags.1?true selective:flags.2?true rows:Vector<KeyboardButtonRow> = ReplyMarkup'

    def __str__(self):
        return '(replyKeyboardMarkup (ID: 0x3502758c) = (resize={}, single_use={}, selective={}, rows={}))'.format(str(self.resize), str(self.single_use), str(self.selective), None if not self.rows else [str(_) for _ in self.rows])



import time
import random
from telethon.utils import BinaryWriter, BinaryReader


class MtProtoPlainSender:
    """MTProto Mobile Protocol plain sender (https://core.telegram.org/mtproto/description#unencrypted-messages)"""
    def __init__(self, transport):
        self._sequence = 0
        self._time_offset = 0
        self._last_msg_id = 0
        self._transport = transport

    def send(self, data):
        """Sends a plain packet (auth_key_id = 0) containing the given message body (data)"""
        with BinaryWriter() as writer:
            writer.write_long(0)
            writer.write_long(self.get_new_msg_id())
            writer.write_int(len(data))
            writer.write(data)

            packet = writer.get_bytes()
            self._transport.send(packet)

    def receive(self):
        """Receives a plain packet, returning the body of the response"""
        seq, body = self._transport.receive()
        with BinaryReader(body) as reader:
            auth_key_id = reader.read_long()
            msg_id = reader.read_long()
            message_length = reader.read_int()

            response = reader.read(message_length)
            return response

    def get_new_msg_id(self):
        """Generates a new message ID based on the current time (in ms) since epoch"""
        # See https://core.telegram.org/mtproto/description#message-identifier-msg-id
        ms_time = int(time.time() * 1000)
        new_msg_id = (((ms_time // 1000) << 32) |  # "must approximately equal unixtime*2^32"
                      ((ms_time % 1000) << 22) |  # "approximate moment in time the message was created"
                      random.randint(0, 524288) << 2)  # "message identifiers are divisible by 4"

        # Ensure that we always return a message ID which is higher than the previous one
        if self._last_msg_id >= new_msg_id:
            new_msg_id = self._last_msg_id + 4

        self._last_msg_id = new_msg_id
        return new_msg_id



from telethon.tl.mtproto_request import MTProtoRequest


class GetWebPagePreviewRequest(MTProtoRequest):
    """Class generated by TLObjects' generator. All changes will be ERASED. Original .tl definition below.
    messages.getWebPagePreview#25223e24 message:string = MessageMedia"""

    # Telegram's constructor ID (and unique identifier) for this class
    constructor_id = 0x25223e24

    def __init__(self, message):
        """
        :param message: Telegram type: «string».
        """
        super().__init__()
        self.result = None
        self.confirmed = True  # Confirmed by default

        self.message = message

    def on_send(self, writer):
        writer.write_int(GetWebPagePreviewRequest.constructor_id, signed=False)
        writer.tgwrite_string(self.message)

    @staticmethod
    def empty():
        """Returns an "empty" instance (all attributes are None)"""
        return GetWebPagePreviewRequest(None)

    def on_response(self, reader):
        self.result = reader.tgread_object()

    def __repr__(self):
        return 'messages.getWebPagePreview#25223e24 message:string = MessageMedia'

    def __str__(self):
        return '(messages.getWebPagePreview (ID: 0x25223e24) = (message={}))'.format(str(self.message))
